package com.sourcelabs.zipper;

import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.MappedByteBuffer;

/**
 * PayloadInputStream
 * <p/>
 * This is a class that takes a fixed set of the zip's ByteBuffer and
 * surfaces it as a stream.
 *
 * @author willpugh  Mar 14, 2006 6:21:45 PM
 */
public class PayloadInputStream extends InputStream {
    final int beginningPos;
    final int endPos;
    final MappedByteBuffer bb;

    public PayloadInputStream(int beginningPos, int length, ByteBuffer bb) {
        this.beginningPos = beginningPos;
        this.endPos = beginningPos + length;
        this.bb = (MappedByteBuffer) bb.duplicate();
        this.bb.order(ByteOrder.LITTLE_ENDIAN);
        this.bb.position(beginningPos);
    }

    public int read() throws IOException {

        int retVal;
        int currPos = bb.position();
        if (currPos >= endPos) {
            retVal = -1;
        } else {
            retVal = bb.get();
        }
        return retVal;
    }

    public int read(byte[] bytes) throws IOException {
        return read(bytes, 0, bytes.length);
    }

    public int read(byte[] bytes, int offset, int length) throws IOException {
        int bytesAvailable = available();
        if (bytesAvailable == 0)
            return -1;

        int bytesToRead = Math.min(length, bytesAvailable);

        bb.get(bytes, offset, bytesToRead);
        return bytesToRead;
    }

    public long skip(long l) throws IOException {
        int currPos = bb.position();
        int bytesToSkip = Math.min(available(), (int) l);
        bb.position(bytesToSkip + currPos);
        return bytesToSkip;
    }

    public int available() throws IOException {
        return endPos - bb.position();
    }

    public void close() throws IOException {
        bb.position(bb.position() + available());
    }
}