package com.sourcelabs.zipper;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.zip.InflaterInputStream;
import java.util.zip.Inflater;

/**
 * @author willpugh  Mar 13, 2006 10:04:38 PM
 */
public class ZipBodyInputStream extends InflaterInputStream {

    public ZipBodyInputStream(int beginningPos, int length, ByteBuffer bb) {
        super(new DeflatedPayloadInputStream(beginningPos, length, bb), new Inflater(true));
    }

    /**
     * DeflatedPayloadInputStream
     *
     * This class extends PayloadInputStream, but tacks on the extra DummyByte that
     * Inflator needs
     *
     */
    static public class DeflatedPayloadInputStream extends PayloadInputStream {
        public DeflatedPayloadInputStream(int beginningPos, int length, ByteBuffer bb) {
            super(beginningPos, length, bb);
        }

        public int read() throws IOException {

            int retVal = super.read();
            if (retVal == -1 && bb.position() == endPos) {
                retVal = 0;
                bb.position(bb.position() + 1);
            }

            return retVal;
        }

        public int read(byte[] bytes, int offset, int length) throws IOException {
            int bytesToRead = length;
            int bytesAvailable = available();
            boolean addDummyByte = false;

            if (bytesAvailable == 0)
                return -1;

            if (bytesToRead >= available()) {
                bytesToRead = available()-1;
                addDummyByte = true;
            }

            bb.get(bytes, offset, bytesToRead);

            if (addDummyByte) {
                bytesToRead++;
                bytes[bytesToRead] = 0;
                bb.position(bb.position() + 1);
            }

            return bytesToRead;
        }

        public int available() throws IOException {
            return super.available() + 1;
        }
    }
}
