package com.sourcelabs.zipper.zip;

import com.sourcelabs.zipper.SignatureMismatchException;
import com.sourcelabs.zipper.zip.ZipFile;

import java.nio.ByteBuffer;

/**
 * @author willpugh  Mar 4, 2006 9:37:05 PM
 */
public class CentralDirHeader {

    public static final int HEADER = 46;

    short       madeByVersion;
    short       versionNeededToExtract;
    short       flags;
    short       compressionFlags;
    short       fileTime;
    short       fileDate;
    int         crc_32;
    int         compressedSize;
    int         uncompressedSize;
    short       fileNameLen;
    short       extraFieldLen;
    short       fileCommentLen;
    short       diskNumberStart;
    short       internalAttrs;
    int         extrenalAttrs;
    int         localHeaderOffset;
    String      fileName;
    byte[]      extra;
    String      fileComment;

    public int load(ByteBuffer buf) throws SignatureMismatchException {

        int signature = buf.getInt();
        if (signature != ZipFile.CENTRAL_DIRECTORY_SIGNATURE) {
            throw new SignatureMismatchException();
        }

        madeByVersion = buf.getShort();
        versionNeededToExtract = buf.getShort();
        flags = buf.getShort();
        compressionFlags = buf.getShort();
        fileTime = buf.getShort();
        fileDate = buf.getShort();
        crc_32 = buf.getInt();
        compressedSize = buf.getInt();
        uncompressedSize = buf.getInt();
        fileNameLen = buf.getShort();
        extraFieldLen = buf.getShort();
        fileCommentLen = buf.getShort();
        diskNumberStart = buf.getShort();
        internalAttrs = buf.getShort();
        extrenalAttrs = buf.getInt();
        localHeaderOffset = buf.getInt();

        byte[]  rawFileName = new byte[fileNameLen];
        buf.get(rawFileName);
        fileName = new String(rawFileName);

        extra = new byte[extraFieldLen];
        buf.get(extra);

        byte[]  rawComment = new byte[fileCommentLen];
        buf.get(rawComment);

        return HEADER + rawFileName.length + extra.length + rawComment.length;
    }

}
