package com.sourcelabs.zipper.zip;

import com.sourcelabs.zipper.InvalidZipFormatException;

/**
 * This is a Zip Entry that was allocated before the central repository was parsed.  This allows
 * us to only get the comments when we need them.
 *
 * @author willpugh  Apr 6, 2006 4:34:56 PM
 */
public class DelayedZipEntry extends ZipEntry {
    ZipFile    file;

    public DelayedZipEntry(String string, ZipFile file) {
        super(string);
        this.file = file;
    }

    DelayedZipEntry(DelayedZipEntry dze) {
        super(dze);
        file = dze.file;
    }

    public void setComment(String string) {
        file = null;
        super.setComment(string);
    }

    public String getComment() {
        if (file != null) {
            ZipIndex index = file.index;
            if (!index.isLoaded()) {
                try {
                    index.load(file.bb);
                } catch (InvalidZipFormatException e) {
                    throw new RuntimeException(e);
                }
            }
            setComment(index.getComment(getName()));
        }
        return super.getComment();
    }

    public Object clone() {
        return new DelayedZipEntry(this);        
    }

}
