package com.sourcelabs.zipper.zip;

import com.sourcelabs.zipper.InvalidZipFormatException;
import com.sourcelabs.zipper.SignatureMismatchException;

import java.nio.ByteBuffer;

/**
 * Subclasses ZipIndexBase, adding the functionality to load an index from the Central Directory
 * into the ZipIndex. 
 *
 * <b>UNDONES</b>
 *   1) UNDONE(willpugh) -- We don't support multi-disk
 * @author willpugh  Mar 4, 2006 2:56:05 PM
 */
public class ZipIndex extends ZipIndexBase {

    private boolean     isLoaded = false;
    private int         centralDirStart = -1;

    public void init(int count) {
        super.init(count);
    }

    public void load(ByteBuffer buf) throws InvalidZipFormatException {
        EndOfCentralDir     eocd = new EndOfCentralDir();
        CentralDirHeader    cdh = new CentralDirHeader();

        int     offset = 0;
        int     oldPos = buf.position();

        try {
            boolean isDone = true;
            do {
                offset = eocd.load(buf, offset);
                buf.position(eocd.centralDirStart);
                setCentralDirStart(eocd.centralDirStart);
                try {
                    isDone = true;
                    cdh.load(buf);
                } catch (SignatureMismatchException e) {
                    isDone = false;
                }

            } while (!isDone);

            //Create our indexes
            init(eocd.numEntries);

            int     currIndex = 0;
            while (currIndex < eocd.numEntries) {

                if (currIndex != 0) {
                    cdh.load(buf);
                }

                offsets[currIndex] = cdh.localHeaderOffset;
                compressedSizes[currIndex] = cdh.compressedSize;
                uncompressedSizes[currIndex] = cdh.uncompressedSize;
                names[currIndex] = cdh.fileName;
                nameMap.put(cdh.fileName, new Integer(currIndex));
                if (cdh.fileComment != null && cdh.fileComment.length() > 0) {
                    addComment(currIndex, cdh.fileComment);
                }
                currIndex++;
            }

        } catch (SignatureMismatchException sme) {
            throw new InvalidZipFormatException("Error: Central Directory is corrupt.", sme);
        } finally {
            buf.position(oldPos);
            setLoaded(true);
        }
    }

    public ZipEntry getEntry(String name, ByteBuffer bb){
        assert(isLoaded());

        ZipEntry    retVal = null;
        Integer     index = (Integer) nameMap.get(name);
        if (index != null) {
            retVal = getEntry(index.intValue(), bb);
        }
        return retVal;
    }

    public ZipEntry getEntry(int index, ByteBuffer bb) {
        assert(isLoaded());

        ZipEntry    retVal = new ZipEntry(names[index]);
        retVal.load(bb, offsets[index], uncompressedSizes[index], compressedSizes[index]);

        return retVal;
    }

    public boolean isLoaded() {
        return isLoaded;
    }

    public void setLoaded(boolean loaded) {
        isLoaded = loaded;
    }

    public int getCentralDirStart() {
        return centralDirStart;
    }

    public void setCentralDirStart(int centralDirStart) {
        this.centralDirStart = centralDirStart;
    }

    public int getFileEntrySize(int i) {        
        return ((i == getCount()-1) ? centralDirStart : getOffset(i+1)) - getOffset(i);
    }
}
