package com.sourcelabs.zipper.zip;

import junit.framework.TestCase;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import com.sourcelabs.zipper.zip.ZipEntryImpl;
import com.sourcelabs.zipper.ZipTestBase;

/**
 * @author willpugh  Mar 17, 2006 10:55:49 AM
 */
public class TestZipHeader extends ZipTestBase {

    public class  TestEntry extends ZipEntryImpl {

        public int testMinDos(long javaTime) {
            int  raw = convertToRawTime(javaTime);
            Calendar rt = convertToJavaTime(raw);
            TestCase.assertEquals(rt.getTimeInMillis(), testRawToJava(DOS_JAN1_1980).getTimeInMillis());
            return raw;
        }

        public Calendar testRawToJava(int raw) {
            Calendar    cal = convertToJavaTime(raw);
            int         rt = convertToRawTime(cal.getTimeInMillis());
            TestCase.assertEquals(raw, rt);
            return cal;
        }

        public int testJavaToRaw(long javaTime) {
            int  raw = convertToRawTime(javaTime);
            Calendar rt = convertToJavaTime(raw);
            TestCase.assertEquals(rt.getTimeInMillis(), javaTime);
            return raw;
        }
    }

    public void testDateTime() {

        TestEntry  th = new TestEntry();
        Date        d = new Date(0);

        int         r = th.testMinDos(d.getTime());
        Calendar    cal = th.testRawToJava(r);

        TestCase.assertEquals(cal.get(Calendar.YEAR), 1980);
        TestCase.assertEquals(cal.get(Calendar.MONTH), 0);
        TestCase.assertEquals(cal.get(Calendar.DAY_OF_MONTH), 1);
        TestCase.assertEquals(cal.get(Calendar.HOUR_OF_DAY), 0);
        TestCase.assertEquals(cal.get(Calendar.MINUTE), 0);
        TestCase.assertEquals(cal.get(Calendar.SECOND), 0);


        Calendar    cal2 = new GregorianCalendar(1984, 3, 23, 9, 30, 32);
        r = th.testJavaToRaw(cal2.getTimeInMillis());
        cal = th.testRawToJava(r);

        TestCase.assertEquals(cal.get(Calendar.YEAR), 1984);
        TestCase.assertEquals(cal.get(Calendar.MONTH), 3);
        TestCase.assertEquals(cal.get(Calendar.DAY_OF_MONTH), 23);
        TestCase.assertEquals(cal.get(Calendar.HOUR_OF_DAY), 9);
        TestCase.assertEquals(cal.get(Calendar.MINUTE), 30);
        TestCase.assertEquals(cal.get(Calendar.SECOND), 32);

        cal2 = new GregorianCalendar(2056, 11, 29, 23, 59, 58);
        r = th.testJavaToRaw(cal2.getTimeInMillis());
        cal = th.testRawToJava(r);

        TestCase.assertEquals(cal.get(Calendar.YEAR), 2056);
        TestCase.assertEquals(cal.get(Calendar.MONTH), 11);
        TestCase.assertEquals(cal.get(Calendar.DAY_OF_MONTH), 29);
        TestCase.assertEquals(cal.get(Calendar.HOUR_OF_DAY), 23);
        TestCase.assertEquals(cal.get(Calendar.MINUTE), 59);
        TestCase.assertEquals(cal.get(Calendar.SECOND), 58);
    }
}
